% Implementation of Ensemble synchronization measure based on Hilbert 
%   transform phase extraction and frobenius norm of the sync matrix

% Please quote: Prasad, P.D., et al., "Single-Trial EEG Classification Using
%  Logistic Regression Based on Ensemble Synchronization," Biomedical and
%  Health Informatics, IEEE Journal of, vol.18, no.3, pp.1074,1080, May 2014
%   DOI: 10.1109/JBHI.2013.2289741

% This algorithm is scalable to any number of channels - 
% example for 4 channels is presented here in this wrapper function.

%setting up the program
clear all;
close all;
clear global;
clc;

%user supplied parameters are to be set here

% set the number of channels - example program
n_ch = 4;

%set the file name for channel 1
file_name1 = 'subject3_ch1.mat';

%set the file name for channel 2
file_name2 = 'subject3_ch2.mat';

%set the file name for channel 3
file_name3 = 'subject3_ch3.mat';

%set the file name for channel 4
file_name4 = 'subject3_ch4.mat';

% %set the file name for channel 5
% file_name5 = 'subject3_ch5.mat';
%
% %set the file name for channel 6
% file_name6 = 'subject3_ch6.mat';

% set sampling frequency in Hz
sampf = 256; % in Hz

% set whether to plot the result
plot_result = 1;

% set output filename
outputfile = 'result.png';

% set frequency band of interest
band = 'lower_gamma';


% load the data from file

%load the file for channel 1
load(file_name1);
disp(['loaded file 1: ' file_name1]);
%load the file for channel 2
load(file_name2);
disp(['loaded file 2: ' file_name2])
%load the file for channel 3
load(file_name3);
disp(['loaded file 3: ' file_name3])
%load the file for channel 4
load(file_name4);
disp(['loaded file 4: ' file_name4])
% %load the file for channel 5
% load(file_name5);
% disp(['loaded file 5: ' file_name5])
% %load the file for channel 6
% load(file_name6);
% disp(['loaded file 6: ' file_name6])

disp ' '

disp(['sampling frequency: ' num2str(sampf) ' Hz'])
disp ' '

% compute the filter response for the frequency band of interest
H_filter = compute_filter(sampf,band);

%call function to compute hilbert phase synchronization
disp 'begin processing data..'
% compute sync for channel pair 1-2
s12 = hilbert_phase_synch(ch1,ch2,sampf,H_filter);
% compute sync for channel pair 1-3
s13 = hilbert_phase_synch(ch1,ch3,sampf,H_filter);
% compute sync for channel pair 1-4
s14 = hilbert_phase_synch(ch1,ch4,sampf,H_filter);
% compute sync for channel pair 2-3
s23 = hilbert_phase_synch(ch2,ch3,sampf,H_filter);
% compute sync for channel pair 2-4
s24 = hilbert_phase_synch(ch2,ch4,sampf,H_filter);
% compute sync for channel pair 3-4
s34 = hilbert_phase_synch(ch3,ch4,sampf,H_filter);
% disp 'finished processing data..'

% obtain the length of the resulting synchronization
len = length(s12);

disp 'computing ensemble synchronization measure..'

% initiate the variable for frobenius norm
fro_norm = zeros(1,len);

for t = 1:len
    % obtain the multi-channel phase synchronization matrix
    ph_sync_matrix = eye(n_ch) ...
        + diag([s12(t) s23(t) s34(t)],1) ...
        + diag([s12(t) s23(t) s34(t)],-1) ... % symmetric in nature - avoid repeated computation
        + diag([s13(t) s24(t)],2) ... 
        + diag([s13(t) s24(t)],-2) ... % symmetric in nature - avoid repeated computation
        + diag(s14(t),3) ... 
        + diag(s14(t),-3) ; % symmetric in nature - avoid repeated computation
    
    % compute the frobenius norm 
    fro_norm(t) = norm(ph_sync_matrix ,'fro');

end

% compute the ensemble synchronization measure
ensemble_measure = sqrt((fro_norm.^2 - n_ch)./(n_ch^2 - n_ch));

% plot the results
disp 'ploting results..'

%draw frobenius measure of ensemble sync
plot((1:length(ensemble_measure))/sampf, ensemble_measure);
hold on;

%graph annotations
grid on;
title('Ensemble synchronization measure based on Frobenius norm');
xlabel('Time (in sec)');
ylabel('Phase Synchronization');

%save the plot
saveas(gcf,outputfile,'png')


%end of program